
#import "DisplayControl.h"

@implementation DisplayControl

- init
{
	const char *sendTypes[2];
	const char *returnTypes[1];

	[super init];

	sendTypes[0] = NXTIFFPboardType;
	sendTypes[1] = NULL;
	returnTypes[0] = NULL;
	[NXApp registerServicesMenuSendTypes:sendTypes
    	andReturnTypes:returnTypes];
	return self;
}

- setUp:(NXBitmapImageRep *)scanImage
{
	NXSize theSize;
	NXRect frame;
	
	theImage = scanImage;
	
	theSize.width = [theImage pixelsWide] + 24.0 ;
	theSize.height = [theImage pixelsHigh] + 58.0 ;

	[window setMaxSize:&theSize];
	
	if (theSize.width > 600)
	{
		theSize.height = theSize.height/(theSize.width/600.0);
		theSize.width = 600;
	}
	[window	sizeWindow:theSize.width :theSize.height];
	[window getFrame:&frame];
	frame.size.width = theSize.width;
	frame.size.height = theSize.height;
	[window placeWindow:&frame];
	[displayView setImage:theImage];
	compression = LZW;
	return self;
}

- copy:sender
{
	[self copyToPasteboard: [Pasteboard new] ];
	return self;
}

- copyToPasteboard:pboard
{
	NXStream *stream;
	char *data;
	int length, maxlen;
	
	NXAtom typelist[1];
	
	typelist[0] = NXTIFFPboardType;
	[pboard declareTypes: typelist num:1 owner:nil];
	
	stream = NXOpenMemory(0, 0, NX_WRITEONLY);
	[theImage writeTIFF:stream
		usingCompression:NX_TIFF_COMPRESSION_NONE
		andFactor:0];
	NXGetMemoryBuffer( stream, &data, &length, &maxlen );
	[pboard writeType: NXTIFFPboardType data: data length: length];
	NXCloseMemory(stream, NX_FREEBUFFER);
	return self;
}

- validRequestorForSendType:(NXAtom)typeSent
                        andReturnType:(NXAtom)typeReturned
{
    if ( (typeSent == NXTIFFPboardType || typeSent == NULL) &&
        ( typeReturned == NULL) )
    {
            return self;
    }
    return nil;
}

- (unsigned int)selectionCount
{
    return 1;
}

- (BOOL) writeSelectionToPasteboard:pboard types:(NXAtom *)types;
{
	[self copyToPasteboard:pboard];
	return YES;
}

- window
{
	return window;
}

- windowWillClose:sender
{
	[theImage free];
	[sender setDelegate:nil];
	[self free];
	return self;
}

- setFilename:(const char *)aFilename
{
	if (filename) free(filename);
	filename = malloc(strlen(aFilename) + 1);
	strcpy(filename, aFilename);
	[window setTitleAsFilename:aFilename];
	return self;
}

- saveAs:sender
{
	id panel;
	const char *dir;
	char *file;
	
	// prompt for a file name & save to that file
	
	if (filename == 0) 
	{
		dir = NXHomeDirectory();
		file = (char *)[window title];
	}
	else
	{
		file = rindex(filename, '/');
		if (file) 
		{
			dir = filename;
			*file = 0;
			file++;
		}
		else
		{
			dir = filename;
			file = (char *)[window title];
		}
	}
	
	compression = LZW;
	panel = [SavePanel new];
	if (!savePanelView)
	{
		[NXApp loadNibSection:"savePanelView.nib" owner:self withNames:NO
			fromZone:[panel zone]];
	}
	
	[panel setRequiredFileType:"tiff"];

	if ( [theImage bitsPerPixel] == 1)
		[panel setAccessoryView:nil];
	else
		[panel setAccessoryView:savePanelView];
	
	if ([panel runModalForDirectory:dir file:file])
	{
		[self setFilename: [panel filename]];
		return [self save:sender];
	}
	return nil;   // didn't save
}

- save:sender
{
	NXStream *theStream;
	
	if (filename == 0) return [self saveAs:sender];
	[window setTitle: "Saving..."];
	
	theStream = NXOpenMemory(NULL, 0, NX_WRITEONLY);  // open unlimited stream

	if ( compression == LZW )
	    [theImage writeTIFF:theStream
		          usingCompression:NX_TIFF_COMPRESSION_LZW
				  andFactor:0];
    else
	if ( compression == JPEG )
	    [theImage writeTIFF:theStream
			      usingCompression:NX_TIFF_COMPRESSION_JPEG
			      andFactor:10.0]; 			// a default to use for now.

	NXSaveToFile(theStream, filename);
	NXCloseMemory(theStream, NX_FREEBUFFER);
	
	[window setTitleAsFilename: filename];
	return self;
}

- changeCompressionType:sender
{
	switch ([sender selectedRow])
	{
	    case 0: compression = LZW; break;
		case 1: compression = JPEG; break;
	}
	return self;
}

- printScan:sender
{
	[displayView printScan:self];
	return self;
}

@end
