
#import "ScanDisplay.h"
#import "mycontentView.h"

@implementation ScanDisplay

- awakeFromNib
{
	[self setVertScrollerRequired: YES];
	[self setHorizScrollerRequired: YES];
	[self setBorderType: NX_LINE];
	[self setDocView: subView];
	[contentView getFrame: &originalFrame];
	[subView setFrame: &originalFrame];
		
	return self;
}

- setImage:(NXBitmapImageRep *)theImage
{
    NXSize imgSize;
	
    displayImage = theImage;
	[subView setImage:theImage];
	[displayImage getSize:&imgSize];

    relativeFactor = [displayImage pixelsHigh]/imgSize.height;
	relativeFactor /= [displayImage pixelsWide]/imgSize.width;
    [contentView scale: relativeFactor : 1.0 ];
    [self setMinMax:self];
	return self;
}

- setScaleFactor:(float)aFactor
{
	if (scaleFactor != aFactor) 
	{
		NXSize newsize;
		float delta = aFactor/scaleFactor;
		scaleFactor = aFactor;
		[contentView scale: delta : delta ];

		[self setMinMax:self];
		[window getMaxSize:&newsize];
		
		if (newsize.width < frame.size.width
			&& newsize.height < frame.size.height)
	      [window sizeWindow:newsize.width - 2 :newsize.height - 36.0];
	}
	
	return self;
}

- changeZoom:sender
{
	[self setScaleFactor: [ [sender selectedCell] tag] / 100.0 ];
	return self;
}

- tile
{
	NXRect scrollerRect, buttonRect;
	
	[super tile];
	
	if ([zoomButton superview] != self)
	{
		[self addSubview:zoomButton];
	}
	
	[hScroller getFrame: &scrollerRect];
	NXDivideRect(&scrollerRect, &buttonRect, 65.0, 2);
	[hScroller setFrame: &scrollerRect];
	NXInsetRect(&buttonRect, 1.0, 1.0);
	[zoomButton setFrame: &buttonRect];
	
	return self;
}

- initFrame:(const NXRect *)theFrame
{
	[super initFrame: theFrame];
	[self setBackgroundGray: NX_WHITE];
	scaleFactor = 1.0;

	return self;
}

- setMinMax:sender
{
	NXSize newsize;
	
	newsize.width = [displayImage pixelsWide]*scaleFactor*relativeFactor + 24.0;
	newsize.height = [displayImage pixelsHigh]*scaleFactor + 58.0;
	
	[window setMaxSize:&newsize];

	newsize.width = 65.0;
	newsize.height = 55.0;
	[window setMinSize:&newsize];

	return self;
}

- printScan:sender
{
    NXSize theSize;
	[[NXApp printInfo] setMarginLeft:0 right:0 top:0 bottom:0];
//	[[NXApp printInfo] setScalingFactor: 1.0/(72.0*scaleFactor)];
    [displayImage getSize:&theSize];
	[[NXApp printInfo] setScalingFactor: 1.0/([displayImage pixelsWide]/theSize.width)];
	[subView printPSCode:nil];

	return self;
}

@end
