
#import "UMAX.h"
#import <stdlib.h>

#define GRAY  (0)
#define RED   (1)
#define GREEN (2)
#define BLUE  (3)
#define COLOR (2)
#define LINEART (4)
#define UNIT_READY (1)

@implementation UMAX

- init
{
    [super init];
    [self loadnib];       // load our nib
    [self findScanner];   // find the scanner
	[self inquiry];		  // get scanner capabilities
	
	// initialize some starting variables
    colorMode = GRAY;
    xresolution = 150;
    yresolution = 150;
    gammaFunction = 0;
	colorWindow = 0;
    theScannedImage = 0;
    return self;
}

- loadnib
{
    NXBundle *myBundle;
    char     nibPath[MAXPATHLEN+1];

    myBundle = [NXBundle bundleForClass:[self class]];

    if ([myBundle getPath:nibPath forResource:"UMAX" ofType:"nib"])
    {
        [NXApp loadNibFile:nibPath owner:self withNames:NO];
    } else 
    {
        NXRunAlertPanel("Error", "Couldn't load UMAX.nib", "OK",
            NULL, NULL );
    }
    [myBundle free];
    return self;
}

- free
{
    close(scannerID);
    return [super free];
}

- changeColorMode:sender
{
    colorMode = [ [sender selectedCell] tag];
	if (colorMode == LINEART ||
	    colorMode == GRAY ) [colorWindowMatrix setEnabled:YES];
	else [colorWindowMatrix setEnabled:NO];
    return self;
}

- setResolution:sender
{
    xresolution = [ [sender selectedCell] tag];
    yresolution = [ [sender selectedCell] tag];
    if (xresolution == 800) xresolution = 400;
    return self;
}

- setGammaFunction:sender
{
    gammaFunction = [ [sender selectedCell] tag];
    return self;
}

- setColorWindow:sender
{
    colorWindow = [ [sender selectedCell] tag];
	return self;
}

- setScannerID:(int)scanner
{
    scannerID = scanner;
    return self;
}

- performScan:(int)xstart :(int)ystart :(int)width :(int)length
     preview:(BOOL)preview
{
    theScannedImage = 0;      // set pointer to image to null
    
    if ([self testUnitReady])
      if ([self reserveUnit])
        if ([self setWindow:xstart :ystart :width :length])
          if ([self scan:preview])     // no preview
            if ([self testUnitReady])
               [self read:xstart :ystart :width :length];

    [self testUnitReady];
    [self objectPosition];
    [self testUnitReady];
    [self releaseUnit];

    return theScannedImage;
}

- performPreview
{
	return [self performScan:0 :0 :10200 :14040 preview:YES];
}

- getDataBufferStatus
{
    NXRunAlertPanel("Error", "getDataBufferStatus not implemented!",
                "OK", NULL, NULL);
    return self;
}

- inquiry
{
    char command = 0x12;
    char retData[148];
    int retLength = 148;
    
    bzero( retData, retLength );
    
    if ( ![self readFromScanner :command :retData :retLength :0])
    {
        NXRunAlertPanel("Error",
                        "No response to inquiry!",
                        "OK", NULL, NULL);
        return(0);
    }
    
    LengthOfDescriptorBlock = 256*retData[0x92] + retData[0x93];

    max_xresolution = (int)retData[0x74] * 100;
	max_yresolution = (int)retData[0x75] * 100;
	
    return self;
}

- objectPosition
{
    struct scsi_req sr;
    struct cdb_10* cdbp = (struct cdb_10 *)&sr.sr_cdb.cdb_c10;
    char returnData;
    u_char cmdBlock = 0x31;    // Operation code 24H
    
    bzero((u_char *)&sr, sizeof(sr));
    
    cdbp->c10_opcode = cmdBlock;

    sr.sr_dma_dir = SR_DMA_RD;  // read type command
    sr.sr_addr = &returnData;   // response into here
    sr.sr_dma_max = 1;          // 1 byte response
    sr.sr_ioto    = 10;         // 10 second timeout

    if ([self DoIoctl:&sr])
    {
        NXRunAlertPanel("Error", "Couldn't send Object Position!",
                        "OK", NULL, NULL);
    }
    
    return self;
}

- read:(int)xstart :(int)ystart :(int)width :(int)length
{
    struct scsi_req sr;
    struct cdb_10* cdbp = (struct cdb_10 *)&sr.sr_cdb.cdb_c10;

    int umaxLength;
    u_char *returnData;
    int bytesPerRow;
    int samplesPerPixel;
    int maxTransferLength = 65535;
    int colorSpace = 0;
    int planar = YES;
    
    char cmdBlock = 0x28;    // Operation code 28H
    
    int pixelwidth = [self calcPixelWidth:xstart :width];
    int pixelheight = [self calcPixelHeight:ystart :length];

    int bitsPerPixel = 8;
    int bitsPerSample = 8;
    
    NXSize imageSize;

    if (colorMode == LINEART) bitsPerSample = 1;
    if (colorMode == GRAY)    bitsPerSample = 8;
    if (colorMode == COLOR)   bitsPerSample = 8;
    
    samplesPerPixel = 1;
    if (colorMode == COLOR) samplesPerPixel = 3;
    
    bitsPerPixel = bitsPerSample * samplesPerPixel;
    bytesPerRow = pixelwidth * bitsPerPixel / 8;
    
    umaxLength = bytesPerRow*pixelheight;
    
    if (colorMode == LINEART || colorMode == GRAY)
    {
        colorSpace = NX_OneIsWhiteColorSpace;
        planar = YES;
    }
    else
    {
        colorSpace = NX_RGBColorSpace;
        planar = NO;
    }
    
    theScannedImage = [[NXBitmapImageRep alloc] initData:NULL
                                   pixelsWide:pixelwidth
                                   pixelsHigh:pixelheight
                                   bitsPerSample:bitsPerSample
                                   samplesPerPixel:samplesPerPixel
                                   hasAlpha:NO
                                   isPlanar:planar
                                   colorSpace:colorSpace
                                   bytesPerRow:bytesPerRow
                                   bitsPerPixel:bitsPerPixel];

    // give the image a proper dpi...

    imageSize.width = pixelwidth * 72.0 / (float)xresolution ;
    imageSize.height = pixelheight * 72.0 / (float)yresolution;
    
    [theScannedImage setSize:&imageSize];
    returnData = [theScannedImage data];

    bzero((char *)returnData, umaxLength);
    bzero((char *)&sr, sizeof(sr));
    
    cdbp->c10_opcode = cmdBlock;
    cdbp->c10_lun    = 0;
    cdbp->c10_mbz2  = 0x00;  

	#ifdef __LITTLE_ENDIAN__
      // cdbp->c10_lba3  = 0;
      // cdbp->c10_lba2  = 0x00;  // image data
      // cdbp->c10_lba1  = 0;
      // cdbp->c10_lba0  = 00;    // window 00
      cdbp->c10_len1   = (maxTransferLength >> 8 ) & 0xff;
      cdbp->c10_len0   = maxTransferLength & 0xff;
    #elif __BIG_ENDIAN__
      // cdbp->c10_lba    = 0;    // image data for window 0
	  #ifdef __NATURAL_ALIGNMENT__
	    cdbp->c10_len[0] = (maxTransferLength >> 8 ) & 0xff;
	    cdbp->c10_len[1] = maxTransferLength & 0xff;
	  #else __NATURAL_ALIGNMENT__
        cdbp->c10_len    = maxTransferLength;
	  #endif
    #endif

    cdbp->c10_ctrl   = 0x00;

    sr.sr_dma_dir = SR_DMA_RD;
    sr.sr_dma_max = maxTransferLength;    /* Number of bytes for device */
    sr.sr_ioto    = 90;        /* I/O timeout in seconds */

	// loop through, reading maxTransferLength at a time.
	
    while (umaxLength > maxTransferLength)
    {
      sr.sr_addr = (char *)returnData;    /* Here is data to write into */

      [self testUnitReady];

      if ([self DoIoctl:&sr])
      {
         NXRunAlertPanel("Error", "Couldn't read!", "OK", NULL, NULL);
         return(0); 
      }
      umaxLength -= maxTransferLength;
      returnData += maxTransferLength;
    }

    if (umaxLength >= 0)
    {
      cdbp->c10_opcode = cmdBlock;
      cdbp->c10_lun    = 0;
	  cdbp->c10_mbz2   = 0;

	  #ifdef __LITTLE_ENDIAN__
        // cdbp->c10_lba3   = 0;
        // cdbp->c10_lba2   = 0x00;  // image data
        // cdbp->c10_lba1   = 0;
        // cdbp->c10_lba0   = 00;  // window 00
        cdbp->c10_len1   = (umaxLength >> 8 ) & 0xff;
        cdbp->c10_len0   = umaxLength & 0xff;
      #elif __BIG_ENDIAN__
        // cdbp->c10_lba    = 0;    // image data & window 0
	    #ifdef __NATURAL_ALIGNMENT__
	      cdbp->c10_len[0] = (umaxLength >> 8 ) & 0xff;
	      cdbp->c10_len[1] = umaxLength & 0xff;
	    #else __NATURAL_ALIGNMENT__
          cdbp->c10_len    = umaxLength;
		#endif
      #endif

      cdbp->c10_ctrl   = 0x00;

      sr.sr_dma_dir = SR_DMA_RD;
      sr.sr_addr = (char *)returnData;    /* Here is data to write into */
      sr.sr_dma_max = umaxLength;            /* Number of bytes for device */
      sr.sr_ioto    = 10;        /* I/O timeout in seconds */

      [self testUnitReady];

      if ([self DoIoctl:&sr])
      {
         NXRunAlertPanel("Error", "Couldn't read!", "OK", NULL, NULL);
         return(0); 
      }
    }

    return self;
}

- releaseUnit
{
    char command = 0x17;
    char retData = 0x00;
    int retLength = 1;
    
    if (![self readFromScanner :command :&retData :retLength :0])
    {
        NXRunAlertPanel("Error",
                        "Trouble with release unit!",
                        "OK", NULL, NULL);
        return(0);
    }
    return self;
}

- requestSense
{
    char command = 0x03;
    char retData[30];
    int retLength = 30;
    
	bzero((char *)retData, 30);
    
    if ( ![self readFromScanner:command :retData :retLength :0] )
    {
        NXRunAlertPanel("Error",
                        "No response to requestSense!\n",
                        "OK", NULL, NULL);
        return(0);
    }
    
    return self;
}

- reserveUnit
{
    char command = 0x16;
    char retData = 0x00;
    int retLength = 1;
    
    if (![self readFromScanner :command :&retData :retLength :0])
    {
        NXRunAlertPanel("Error",
                        "No response to reserveUnit!",
                        "OK", NULL, NULL);
        return(0);
    }
    return self;
}

- send
{
    return self;
}

- setScannerUnit
{
    return self;
}

- setWindow:(int)xstart :(int)ystart
           :(int)width  :(int)length
{
    // max ystart+length = 14040 (11.7 inches * 1200)
    // max xstart+width = 10200 (8.5 inches * 1200)
    
    struct scsi_req sr;
    struct cdb_10* cdbp = (struct cdb_10 *)&sr.sr_cdb.cdb_c10;
    u_char* WriteDataBuffer = 0;
    u_char* winGrayDescriptorBlock = 0;
    u_char* winRedDescriptorBlock = 0;
    u_char* winGrnDescriptorBlock = 0;
    u_char* winBluDescriptorBlock = 0;
    u_char *winParmDataBlock = 0;
    char message[80];
    u_char cmdBlock = 0x24;    // Operation code 24H
    int umaxLength;

    if (colorMode == GRAY || colorMode == LINEART) // (gray or lineart)
        umaxLength = LengthOfDescriptorBlock + 8;
    else
        umaxLength = 3*LengthOfDescriptorBlock + 8; 
        
    WriteDataBuffer = (u_char *)calloc(umaxLength, sizeof(u_char));
    
    if (xstart+width > 8.5*1200 || ystart+length > 11.7*1200)
    {
        sprintf(message, "Can't scan these dimensions.\nMax dims = (%d,%d)",
                (int)(8.5*1200), (int)(11.7*1200));
        NXRunAlertPanel("Error", message, "OK", NULL, NULL);
        return (0);
    }

    bzero((u_char *)&sr, sizeof(sr));
    bzero((u_char *)WriteDataBuffer, umaxLength);
    
    winParmDataBlock = WriteDataBuffer;
    winGrayDescriptorBlock = WriteDataBuffer + 8;
    if (colorMode == COLOR)
    {
       winRedDescriptorBlock = WriteDataBuffer + 8;
       winGrnDescriptorBlock = WriteDataBuffer + LengthOfDescriptorBlock + 8;
       winBluDescriptorBlock = WriteDataBuffer +2*LengthOfDescriptorBlock + 8;
    }
    
    winParmDataBlock[6] = 0;
    if (colorMode == GRAY || colorMode == LINEART)
        winParmDataBlock[7] = LengthOfDescriptorBlock;
    else
        winParmDataBlock[7] = 3*LengthOfDescriptorBlock;
    
    if (colorMode == LINEART || colorMode == GRAY)
        [self setWinDescriptorBlock :colorMode :xstart :ystart :width :length
                    :winGrayDescriptorBlock];
    else
    {
        [self setWinDescriptorBlock :RED :xstart :ystart :width :length
                    :winRedDescriptorBlock];
        [self setWinDescriptorBlock :GREEN :xstart :ystart :width :length
                    :winGrnDescriptorBlock];
        [self setWinDescriptorBlock :BLUE :xstart :ystart :width :length
                    :winBluDescriptorBlock];
    }
    
    // First, a read type operation to prep the scanner.
        
    cdbp->c10_opcode = cmdBlock;
    cdbp->c10_lun    = 0;
    cdbp->c10_mbz2   = 0;
	#ifdef __LITTLE_ENDIAN__
      // cdbp->c10_lba3 = 0;
      // cdbp->c10_lba2 = 0;
      // cdbp->c10_lba1 = 0;
      // cdbp->c10_lba0 = 0;
      cdbp->c10_len1 = (umaxLength >> 8 ) & 0xff;
      cdbp->c10_len0 = umaxLength & 0xff;
    #elif __BIG_ENDIAN__
      // cdbp->c10_lba  = 0;
	  #ifdef __NATURAL_ALIGNMENT__
	    cdbp->c10_len[0] = (umaxLength >> 8 ) & 0xff;
	    cdbp->c10_len[1] = umaxLength & 0xff;
	  #else __NATURAL_ALIGNMENT__
        cdbp->c10_len  = umaxLength;
	  #endif
    #endif

    cdbp->c10_ctrl = 0x00;

    sr.sr_dma_dir = SR_DMA_WR;
    sr.sr_addr = WriteDataBuffer;  /* Here is data to write into */
    sr.sr_dma_max = umaxLength;            /* Number of bytes for device */
    sr.sr_ioto    = 10;        /* I/O timeout in seconds */

    if ([self DoIoctl:&sr])
    {
        NXRunAlertPanel("Error", "Couldn't send set window!",
                        "OK", NULL, NULL);
		free( WriteDataBuffer );
		return(0);
    }
    
    free( WriteDataBuffer );
    return self;
}

- (int)testUnitReady
{
    char command = 0x00;
    char retData  = 0x00;

    int unitReady = !UNIT_READY;
    int counter = 0;
    
    // we must get a good response, so we loop.
    
    while( !unitReady && counter < 600)
    {
        if (counter > 0) usleep(250000);      // sleep 25 msec (1/4 sec)
        unitReady = [self readFromScanner:command :&retData :1 :0];
        counter++;
    }
    
    if ( counter == 600 )
    {
        NXRunAlertPanel("Error", "Received no reply to testUnitReady!",
                 "OK", NULL, NULL);
        return(!UNIT_READY);
    }

    return UNIT_READY;
}

- writeBuffer
{
    return self;
}

- scan:(BOOL)preview
{
    char retData = 0x0;
    int retLength = 0;   // no return (requiring return hangs black)
    char ctrl = 0x0;
    
    // scan has to be handled special.
    // opcode = 0x1BH
    // byte#5 is:  bit 7 = preview bit
    //         bit 6 = auto feed/flatbed flag (=0 is flatbed)
    //         bit 5 = quality or speed calibration (1 or 0)
    // byte#5 is the c6s_ctrl byte.

    char cmdBlock = 0x1B;    // Operation code 1BH
    if (preview) ctrl = ctrl | 0x80;   // preview
    // if (calibration) ctrl = ctrl | 0x20;

    if ( ![self readFromScanner :cmdBlock :&retData :retLength :ctrl] )
    {
       NXRunAlertPanel( "Error",
                           "Couldn't scan (preview) area!",
                        "OK", NULL, NULL );
       return(0); 
    }
    return self;
}

- (int)readFromScanner:(u_char)opcode
                 :(u_char *)returnDataPtr
                 :(int)Length
                 :(u_char)ctrl
{
    struct scsi_req sr;                /* Used with ioctl SGIOCREQ */
    struct cdb_6s* cdbp = &sr.sr_cdb.cdb_c6s;

    // zero the scsi request block
    bzero(&sr, sizeof(struct scsi_req));

    cdbp->c6s_opcode = opcode;
    cdbp->c6s_ctrl   = ctrl;
    cdbp->c6s_lun = 0;
    #ifdef __LITTLE_ENDIAN__
        cdbp->c6s_len0 = Length & 0xff;
        cdbp->c6s_len1 = ( Length >> 8 ) & 0xff;
        cdbp->c6s_len2 = ( Length >> 16 ) & 0xff;
    #elif __BIG_ENDIAN__
	  #ifdef __NATURAL_ALIGNMENT__
	    cdbp->c6s_len[0] = (Length >> 16 ) & 0xff;
	    cdbp->c6s_len[1] = (Length >> 8 ) & 0xff;
	    cdbp->c6s_len[2] = Length & 0xff;
	  #else __NATURAL_ALIGNMENT__
        cdbp->c6s_len = Length;        /* Number of bytes for device */
	  #endif
    #endif

    sr.sr_dma_dir = SR_DMA_RD;
    sr.sr_addr = (char *)returnDataPtr;     // Here is data to write
    sr.sr_dma_max = Length;                 // Number of bytes for device
    sr.sr_ioto    = 30;                     // I/O timeout in seconds

    // DoIoctl returns a zero for a good return. "if" statements take
    // zero as a false, and not zero as a true. Not zero is a bad return.
    // So the test is easy. Our return is reversed, though. Sigh...
    // Ends up this way because of the scsi status return, 0 being good.
    
    if ([self DoIoctl:&sr] != 0)
    { 
        return(0);  // return a "false" -> bad return value.
    }
    return(1);    // return a 1 for a good return from readFromScanner
}

- (int)DoIoctl:(struct scsi_req *)Request
{
    int I;
    const char *IOStatusMsgs[25] = {    /* Values from 
                                            /usr/include/nextdev/scsireg.h */
        "Successful",                        /* SR_IOST_GOOD    */
        "Selection timeout",                 /* SR_IOST_SELTO   */
        "Check status, sr_esense valid",     /* SR_IOST_CHKSV   */
        "Check status, sr_esense not valid", /* SR_IOST_SHKSNV  */
        "Tried to move > sr_dma_max",        /* SR_IOST_DMAOR   */
        "Sr_ioto exceeded",                  /* SR_IOST_IOTO    */
        "Bus violation",                     /* SR_IOST_BV      */
        "Command reject",                    /* SR_IOST_CMDREJ  */
        "Memory allocation failure",         /* SR_IOST_MEMALL  */
        "Memory fault",                      /* SR_IOST_MEMF    */
        "Not super user",                    /* SR_IOST_PERM    */
        "Device not open",                   /* SR_IOST_NOPEN   */
        "Target aborted command",            /* SR_IOST_TABT    */
        "Bad SCSI status byte",              /* ST_IOST_BADST   */
        "Internal driver error",             /* ST_IOST_INT     */
        "Unexpected byte count",             /* SR_IOST_BCOUNT  */
        "Desired volume not available",      /* SR_IOST_VOLNA   */
        "Media Write Protected",             /* SR_IOST_WP      */
        "DMA alignment error",               /* SR_IOST_ALIGN   */
        "Mach IPC failure",                  /* SR_IOST_IPCFAIL */
        "Bus was reset during processing",   /* SR_IOST_RESET   */
        "SCSI Bus Parity Error",             /* SR_IOST_PARITY  */
        "Gross Hardware Failure",            /* SR_IOST_HW      */
        "DMA error",                         /* SR_IOST_DMA     */
        "?"
        };
    const char *SenseMsgs[12] = {        /* Values from
                                            /usr/include/nextdev/scsireg.h */
        "No sense to report",            /* SENSE_NOSENSE        */
        "Recovered error",               /* SENSE_RECOVERED      */
        "Not ready",                     /* SENSE_NOTREADY       */
        "Media flaw",                    /* SENSE_MEDIA          */
        "H/W failure",                   /* SENSE_HARDWARE       */
        "Illegal request",               /*  */
        "Unit attention",                /* SENSE_UNITATTENTION  */
        "Data access protected",         /* SENSE_DATAPROTECT    */
        "Target aborted commant",        /* SENSE_ABORTEDCOMMAND */
        "Some data not transfered",      /* SENSE_VOLUMEOVERFLOW */
        "Source/media data mismatch",    /* SENSE_MISCOMPARE     */
        "?"
        };
    char message[200];
    
    if (ioctl(scannerID,SGIOCREQ, Request) < 0)
    {
        sprintf(message, "Error issuing ioctl to scanner. Error number = %d.",
            errno);
        NXRunAlertPanel("Error", message, "OK", NULL, NULL);
        return(1);                    // Return an error.
    }

    if (Request->sr_io_status == SR_IOST_GOOD) return(0);   /* success */

    // ioctl failed. Report it if it was _not_ a test unit ready command.
    
    if (Request->sr_cdb.cdb_c6s.c6s_opcode != 0x00)
    {
        I = Request->sr_io_status;
        if (I >25-1) I = 25-1;
        sprintf(message, "SCSI error: sr_io_status=%02XX (%s)\n",
            Request->sr_io_status, IOStatusMsgs[I]);
        NXRunAlertPanel("Error", message, "OK", NULL, NULL);
        if(Request->sr_io_status == SR_IOST_CHKSV)
        {
            I = Request->sr_esense.er_sensekey;
            if (I >12-1) I = 12-1;
    
            sprintf(message,
                "Report failed!\nSense key=%02XX  (%s)\nSense code = %02XX",
                Request->sr_esense.er_sensekey, SenseMsgs[I],
                Request->sr_esense.er_addsensecode);
                [self DumpScsi_Req:Request];            
            NXRunAlertPanel("Error", message, "OK", NULL, NULL);                
        }
    }
    return(Request->sr_io_status);       // return status (0 is good)
}

- setWinDescriptorBlock:(int)window
      :(int)xupperleft   :(int)yupperleft
      :(int)width        :(int)length
      :(u_char *)w_ptr
{
    int tbrightness = [brightness intValue];
    int tcontrast = 128;

    bzero( w_ptr, LengthOfDescriptorBlock );
    
    w_ptr[0] = window;              // window
    if (window == LINEART) w_ptr[0] = GRAY;
    w_ptr[2] = xresolution >> 8 ;      // You can't use the value larger  
                                     // than the value Inquiry return
    w_ptr[3] = xresolution & 0xff ;  // XResolution & 0xFF;
    w_ptr[4] = yresolution >> 8 ;    // YResolution >> 8;
    w_ptr[5] = yresolution & 0xff ;  // YResolution & 0xFF;
    w_ptr[8] = xupperleft >> 8;      // TopLeft position (1200,1200),that means
    w_ptr[9] = xupperleft & 0xFF;    // position (1,1,) in inch. 
    w_ptr[12] = yupperleft >> 8;
    w_ptr[13] = yupperleft & 0xFF;
    w_ptr[16] = width >> 8;          // Width  of frame in 1200 dpi 
    w_ptr[17] = width & 0xFF;
    w_ptr[20] = length >> 8;         // Length of frame in 1200 dpi
    w_ptr[21] = length & 0xFF;
    w_ptr[22] = tbrightness;          // Not used in Gray Mode
    w_ptr[23] = ~w_ptr[22] + 1;      // 2's complement: Not used in Gray Mode
    w_ptr[24] = tcontrast;            // No used in Gray Mode 

    if (window == LINEART)
        w_ptr[25] = 0;
    else if (window == GRAY)
        w_ptr[25] = 2;
    else
        w_ptr[25] = 5;               // rgb
        
    if (window == LINEART)
        w_ptr[26] = 1;               // 1 bpp
    else
        w_ptr[26] = 8;               // 8bpp

    w_ptr[27] = 0;                   // half-tone pattern 
    w_ptr[28] = 3;                   // should be ignored (not supported)

    if (window == GRAY || window == RED || window == GREEN || window == BLUE)
       w_ptr[29] = 0x03;
    if (window == LINEART)
       w_ptr[29] = 0x83;
    w_ptr[40] = 1;                     // Scan Speed (fast, no smearing)

    if (window == RED)
        w_ptr[41] = 0x80;
    else if (window == GREEN)
        w_ptr[41] = 0x40;
    else if (window == BLUE)
        w_ptr[41] = 0x20;
    else
        w_ptr[41] = colorWindow;     // 0x00;            // gray

    w_ptr[42] = 255;                 // highlight
    w_ptr[43] = 0;                     // shadow

    if (gammaFunction == 0)
        w_ptr[46] = 0x0f;            // normal curve
    else
        w_ptr[46] = gammaFunction;           // built-in curves
        
    w_ptr[47] = 0x11;                 // flatbed scanner
    w_ptr[48] = 0x01;                 // cbhs type (0-255)

    if (window == LINEART)
        w_ptr[57] = 0x0f;
    else
    if (window == GRAY )
        w_ptr[57] = 0x0d;             // gray calibration
    else
        w_ptr[57] = 0x06;             // rgb calibration
        
    w_ptr[58]= 1;                     // color order - line w/no ccd distance
    w_ptr[59]= 0;
    w_ptr[60]= 0;
     
    return self;
}

- findScanner
{
    struct scsi_adr sa;
    struct scsi_req sr;
    struct cdb_6 *cdbp = &sr.sr_cdb.cdb_c6;

    char returnData[148];           // Read the inquiry response into here.
    int target;
    
    // Can we open the generic scsi device? If not, we pop up a
    // panel and then quit.
    
    if ((scannerID = open("/dev/sg0", O_RDWR, 0666)) <= 0)
    {
        NXRunAlertPanel("Error",
                        "Unable to open scanner device /dev/sg0",
                        "OK", NULL, NULL);
        exit(-1);
    }
    
    // Start scanning through all the scsi devices. Assume (of course)
    // that the scanner will be at LUN 0.
    
    for(target = 0; target <= 7; target++)
    {
        sa.sa_target = target;
        sa.sa_lun = 0;

        // if a device is present at the target number....
        if (ioctl(scannerID, SGIOCSTL, &sa) >= 0)
        {
            bzero(&sr, sizeof(struct scsi_req));

            cdbp->c6_opcode = C6OP_INQUIRY;     // Sending inquiry command
            cdbp->c6_lun = 0;                   // Assume LUN = 0
            cdbp->c6_len = sizeof(returnData);  // Number of bytes from device

            sr.sr_dma_dir = SR_DMA_RD;            // It's a read function
            sr.sr_addr    = (char *) &returnData; // with data ending up here.
            sr.sr_dma_max = cdbp->c6_len;         // and it's this long.
            sr.sr_ioto    = 2;                    // Should respond quickly.

            // If the device responded adequately...
            
            if (ioctl(scannerID, SGIOCREQ, &sr) >= 0)
            {
                // returning good status, check to see if it's our scanner.
                if(!sr.sr_io_status)
                {
// Disabled for now so other can try other types of scanners.
//                    if (!strncmp("UMAX", &returnData[8],4) )
//                    {
//                        // Enable auto-sense if error.
//                         ioctl(scannerID, SGIOCENAS, &sa);
//                        return self;
//                    }
				    if ( (returnData[0x00] & 0x1F) == DEVTYPE_SCANNER)
					{
					  return self;
					}
                }
            }
        }
    }

    NXRunAlertPanel("Error","No supported scanner found!", "OK", NULL, NULL);

    return self;
}

- DumpScsi_Req:(struct scsi_req *)Request
{
    int i;

    #if __BIG_ENDIAN__
    #else
       fprintf(stderr, "\tsize=%d\n", Request->sr_cdb_length);
       fprintf(stderr, "\tCDB=%s\n", 
	   		[self Hex:(char *)Request :Request->sr_cdb_length]);
    #endif

    fprintf(stderr, "\tsr_dma_dir=%d\n", Request->sr_dma_dir);
    fprintf(stderr, "\tsr_addr=%8.8x\n", (int)Request->sr_addr);
    fprintf(stderr, "\tsr_dma_max=%d\n", Request->sr_dma_max);
    fprintf(stderr, "\tsr_ioto=%d\n", Request->sr_ioto);
    fprintf(stderr, "\tsr_io_status=%d\n", Request->sr_io_status);
    fprintf(stderr, "\tsr_scsi_status=%02x\n", Request->sr_scsi_status);
    fprintf(stderr, "\tsr_esense=%s\n", [self Hex:(char *)&(Request->sr_esense) 
                        :sizeof (struct esense_reply)]);
    fprintf(stderr, "\tsr_dma_xfr=%d\n", Request->sr_dma_xfr);
    fprintf(stderr, "\tsr_exec_time=%s\n",
		[self Hex:(char *)&(Request->sr_exec_time) :sizeof(struct timeval)]);

	for (i=0; i<=14; i++)
	{
		printf("Sense: %2d (0x%02X) = 0x%s", i, i,
                    [self Hex:(u_char *)&(Request->sr_esense)+i :1]);
		printf("         %2d (0x%02X) = 0x%s\n", i+14+1, i+14+1,
                    [self Hex:(u_char *)&(Request->sr_esense)+i+14+1 :1]);
	}

    return self;
}

- (char *)Hex :(char *)String :(int) Length
{
	const char HexChar[16] = "0123456789ABCDEF";
	char _HexString[4096];          /* Hex builds into here */
	int I;
	char *P = &_HexString[0];               /* Return data goes here */

	for (; Length; Length--)
	{
		I = (*String>>4) & 15;
		*P = HexChar[I];
		P++;
		I = *String & 15;
		*P = HexChar[I];
		P++;
		String++;
	}
	*P = '\0';
	return(_HexString);
}

- (int)calcPixelWidth:(int)xstart :(int)width
{
    // calculate the number of missing pixels from the scan width.
        
    int numDet = width/3;      // 1200/400 = 3 (must change for other scan)
    int missingDet = 0;
    int numMissing = 0;
    int detNum = 1;
    
//    if (xresolution == 400)
    if (xresolution == max_xresolution)
    {
        numMissing = 0;
    }
    else
    {
        while (missingDet <= numDet)
        {
//            missingDet = detNum * 400/ (400 - xresolution);
            missingDet = detNum * 
			      max_xresolution/ (max_xresolution - xresolution);
            detNum++;
        }
        numMissing = detNum - 2;    // -1 for detNum++, -1 for start of 1
    }
    
    return numDet - numMissing;        
}

- (int)calcPixelHeight:(int)ystart :(int)length
{
    // we calculate the length the same way as the width
    
    int numDet;
    int missingDet = 0;
    int numMissing = 0;
    int detNum = 1;
//    int scanres = 400;
	int scanres = max_yresolution / 2;
	    
//    if (yresolution > 400) scanres = 800;
    if (yresolution > max_yresolution/2) scanres = max_yresolution;
    numDet = (length*scanres)/1200;
    
    if (yresolution == scanres)
    {
        numMissing = 0;
    }
    else
    {
        while (missingDet <= numDet)
        {
            missingDet = detNum * scanres/ (scanres - yresolution);
            detNum++;
        }
        numMissing = detNum - 2;    // -1 for detNum++, -1 for start of 1
    }
    
    return numDet - numMissing;        
}

@end
